import { buildSchema } from 'graphql';
import { diff } from '@graphql-inspector/core';
import { createConfig, } from './helpers/config.js';
import { loadSources } from './helpers/loaders.js';
import { createLogger } from './helpers/logger.js';
import { notifyWithDiscord, notifyWithSlack, notifyWithWebhook } from './helpers/notifications.js';
export async function handleSchemaChangeNotifications({ context, ref, repo, owner, before, loadFile, loadConfig, onError, release, action, }) {
    const id = `${owner}/${repo}#${ref}`;
    const logger = createLogger('NOTIFICATIONS', context, release);
    logger.info(`started - ${id}`);
    logger.info(`action - ${action}`);
    const isBranchPush = ref.startsWith('refs/heads/');
    if (!isBranchPush) {
        logger.warn(`Received Push event is not a branch push event (ref "${ref}")`);
        return;
    }
    const rawConfig = await loadConfig();
    if (!rawConfig) {
        logger.error(`Missing config file`);
        return;
    }
    const branch = ref.replace('refs/heads/', '');
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    const config = createConfig(rawConfig, () => { }, [branch]);
    if (!config.notifications) {
        logger.info(`disabled. Skipping...`);
        return;
    }
    logger.info(`enabled`);
    if (config.branch !== branch) {
        logger.info(`Received branch "${branch}" doesn't match expected branch "${config.branch}". Skipping...`);
        return;
    }
    const oldPointer = {
        path: config.schema,
        ref: before,
    };
    const newPointer = {
        path: config.schema,
        ref,
    };
    const sources = await loadSources({
        config,
        oldPointer,
        newPointer,
        loadFile,
    });
    const schemas = {
        old: buildSchema(sources.old, {
            assumeValid: true,
            assumeValidSDL: true,
        }),
        new: buildSchema(sources.new, {
            assumeValid: true,
            assumeValidSDL: true,
        }),
    };
    logger.info(`built schemas`);
    const changes = await diff(schemas.old, schemas.new);
    if (!changes.length) {
        logger.info(`schemas are equal. Skipping...`);
        return;
    }
    const notifications = config.notifications;
    async function actionRunner(target, fn) {
        try {
            await fn();
        }
        catch (error) {
            onError(error);
            logger.error(`Failed to send a notification via ${target}`, error);
        }
    }
    if (hasNotificationsEnabled(notifications)) {
        const actions = [];
        const commit = context.payload.commits?.[0]?.id;
        if (notifications.slack) {
            actions.push(actionRunner('slack', () => notifyWithSlack({
                url: notifications.slack,
                changes,
                environment: config.name,
                repo,
                owner,
                commit,
            })));
        }
        if (notifications.discord) {
            actions.push(actionRunner('discord', () => notifyWithDiscord({
                url: notifications.discord,
                changes,
                environment: config.name,
                repo,
                owner,
                commit,
            })));
        }
        if (notifications.webhook) {
            actions.push(actionRunner('webhook', () => notifyWithWebhook({
                url: notifications.webhook,
                changes,
                environment: config.name,
                repo,
                owner,
                commit,
            })));
        }
        if (actions.length) {
            await Promise.all(actions);
        }
    }
}
function hasNotificationsEnabled(notifications) {
    return notifications && typeof notifications === 'object';
}
